@@ -7,7 +7,7 @@
 ##----------------------------------------------------------------------
 from django.shortcuts import get_object_or_404
 from django.utils.html import escape
-from noc.lib.app import ModelApplication,URL,HasPerm
+from noc.lib.app import ModelApplication,URL,HasPerm,view
 from noc.cm.models import Object
 from noc.lib.highlight import NOCHtmlFormatter
 from pygments.lexers import DiffLexer
