@@ -8,11 +8,12 @@
 from django.db import models
 from django.db.models import Q
 from noc.main.models import Language,database_storage
-from noc.main.menu import Menu
 from django.contrib.auth.models import User
 from noc.kb.parsers import parser_registry
-from noc.main.search import SearchResult
+from noc.lib.search import SearchResult
 from noc.lib.validators import is_int
+from noc.lib.app.site import site
+from noc.lib.fields import AutoCompleteTagsField
 import difflib,re
 
 ##
@@ -20,17 +21,6 @@ import difflib,re
 ##
 parser_registry.register_all()
 ##
-## KB Categories
-##
-class KBCategory(models.Model):
-    class Meta:
-        verbose_name="KB Category"
-        verbose_name_plural="KB Categories"
-        ordering=("name",)
-    name=models.CharField("Name",max_length=64,unique=True)
-    def __unicode__(self):
-        return self.name
-##
 ## KB Entry
 ##
 class KBEntry(models.Model):
@@ -42,12 +32,15 @@ class KBEntry(models.Model):
     body=models.TextField("Body")
     language=models.ForeignKey(Language,verbose_name="Language",limit_choices_to={"is_active":True})
     markup_language=models.CharField("Markup Language",max_length="16",choices=parser_registry.choices)
-    categories=models.ManyToManyField(KBCategory,verbose_name="Categories",null=True,blank=True)
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     def __unicode__(self):
         if self.id:
             return u"KB%d: %s"%(self.id,self.subject)
         else:
             return u"New: %s"%self.subject
+    ##
+    def get_absolute_url(self):
+        return site.reverse("kb:view:view",self.id)
     ## Wiki parser class
     def _parser(self):
         return parser_registry[self.markup_language]
@@ -56,11 +49,6 @@ class KBEntry(models.Model):
     def _html(self):
         return self.parser.to_html(self)
     html=property(_html)
-    #
-    def view_link(self):
-        return "<A HREF='/kb/%d/'>View</A>"%self.id
-    view_link.short_description="View"
-    view_link.allow_tags=True
     ##
     ## Search engine
     ##
@@ -78,7 +66,8 @@ class KBEntry(models.Model):
                     text=r.body[idx-100:idx+len(query)+100]
                 except ValueError:
                     text=r.body[:100]
-                yield SearchResult(url="/kb/%d/"%r.id,
+                yield SearchResult(
+                    url=("kb:view:view",r.id),
                     title="KB%d: %s"%(r.id,r.subject),
                     text=text,
                     relevancy=1.0)
@@ -127,7 +116,7 @@ class KBEntry(models.Model):
     ## Returns a list of visible attachments
     ##
     def _visible_attachments(self):
-        return [{"name":x.name,"size":x.size,"mtime":x.mtime,"url":x.url,"description":x.description}
+        return [{"name":x.name,"size":x.size,"mtime":x.mtime,"description":x.description}
             for x in self.kbentryattachment_set.filter(is_hidden=False).order_by("name")]
     visible_attachments=property(_visible_attachments)
     ##
@@ -218,12 +207,6 @@ class KBEntryAttachment(models.Model):
             return None
     mtime=property(_mtime)
     ##
-    ## Download URL
-    ##
-    def _url(self):
-        return "/kb/%d/attachment/%s/"%(self.kb_entry.id,self.name)
-    url=property(_url)
-    ##
     ## Search engine
     ##
     @classmethod
@@ -231,7 +214,8 @@ class KBEntryAttachment(models.Model):
         if user.has_perm("kb.change_kbentry"):
             q=Q(name__icontains=query)|Q(description__icontains=query)
             for r in KBEntryAttachment.objects.filter(q):
-                yield SearchResult(url="/kb/%d/"%r.kb_entry.id,
+                yield SearchResult(
+                    url=("kb:view:view",r.kb_entry.id),
                     title="KB%d: %s"%(r.kb_entry.id,r.kb_entry.subject),
                     text="Attachement: %s (%s)"%(r.name,r.description),
                     relevancy=1.0)
@@ -296,9 +280,12 @@ class KBEntryTemplate(models.Model):
     body=models.TextField("Body")
     language=models.ForeignKey(Language,verbose_name="Language",limit_choices_to={"is_active":True})
     markup_language=models.CharField("Markup Language",max_length="16",choices=parser_registry.choices)
-    categories=models.ManyToManyField(KBCategory,verbose_name="Categories",null=True,blank=True)
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     def __unicode__(self):
         return self.name
+    
+    def get_absolute_url(self):
+        return site.reverse("kb:kbentrytemplate:change",self.id)
     ##
     ## Returns template variables list
     ##
@@ -307,22 +294,3 @@ class KBEntryTemplate(models.Model):
         var_set.update(rx_template_var.findall(self.body))
         return sorted(var_set)
     var_list=property(_var_list)
-
-##
-## Application menu
-##
-class AppMenu(Menu):
-    app="kb"
-    title="Knowledge Base"
-    items=[
-        ("Knowledge Base", "/kb/", "is_logged_user()"),
-        ("New from Template", "/kb/template/", "is_logged_user()"),
-        ("Setup",[
-            ("Categories","/admin/kb/kbcategory/", "kb.change_kbcategory"),
-            ("Entries",   "/admin/kb/kbentry/",    "kb.change_kbentry"),
-            ("Global Bookmarks", "/admin/kb/kbglobalbookmark/", "kb.change_kbglobalbookmark"),
-            ("User Bookmarks",  "/admin/kb/kbuserbookmark/",    "is_logged_user()"),
-            ("Templates",   "/admin/kb/kbentrytemplate/",    "kb.change_kbentrytemplate"),
-        ])
-    ]
-
