@@ -154,7 +154,11 @@ def format_frames(frames):
         r+=[format_source(f["lineno"]+1,f["post_context"])]
         r+=["Variables:"]
         for n,v in f["vars"]:
-            r+=["%20s = %s"%(n,repr(v))]
+            try:
+                v=repr(v)
+            except:
+                v="repr() failed"
+            r+=["%20s = %s"%(n,v)]
         r+=["-"*72]
     r+=["END OF TRACEBACK"]
     return "\n".join(r)
@@ -183,9 +187,12 @@ def error_report():
         if DEBUG_CTX_SET_UID: # Change crashinfo userid to directory's owner
             os.chown(p,DEBUG_CTX_SET_UID,-1)
 
-def frame_report(frame):
+def frame_report(frame,caption=None):
     now=datetime.datetime.now()
-    r=["EXECUTION FRAME REPORT (%s)"%str(now)]
+    r=[]
+    if caption:
+        r+=[caption]
+    r+=["EXECUTION FRAME REPORT (%s)"%str(now)]
     r+=["Working directory: %s"%os.getcwd()]
     r+=[format_frames(get_execution_frames(frame))]
     logging.error("\n".join(r))
