@@ -449,7 +449,11 @@ class ConnectedTCPSocket(TCPSocket):
         if self.local_address:
             self.socket.bind((self.local_address,0))
         e=self.socket.connect_ex((self.address,self.port))
-        if e not in (0, EISCONN,EINPROGRESS, EALREADY, EWOULDBLOCK):
+        if e in (ETIMEDOUT, ECONNREFUSED):
+            self.on_conn_refused()
+            self.close()
+            return
+        elif e not in (0, EISCONN,EINPROGRESS, EALREADY, EWOULDBLOCK):
             raise socket.error, (e, errorcode[e])
         super(ConnectedTCPSocket,self).create_socket()
         
@@ -580,6 +584,7 @@ class FileWrapper(object):
 ##
 class PTYSocket(Socket):
     def __init__(self,factory,argv):
+        self.pid=None
         super(PTYSocket,self).__init__(factory)
         self.argv=argv
         self.out_buffer=""
@@ -592,7 +597,7 @@ class PTYSocket(Socket):
         else:
             self.socket=FileWrapper(fd)
             super(PTYSocket,self).create_socket()
-            
+    
     def handle_read(self):
         try:
             data=self.socket.read(8192)
@@ -622,6 +627,8 @@ class PTYSocket(Socket):
     
     def close(self):
         Socket.close(self)
+        if self.pid is None:
+            return
         try:
             pid,status=os.waitpid(self.pid,os.WNOHANG)
         except os.error:
