@@ -9,7 +9,9 @@ from south.db import db
 from noc.main.models import *
 
 class Migration:
-    
+    depends_on=(
+        ("sa","0003_task_schedule"),
+    )
     def forwards(self):
         db.execute("UPDATE sa_taskschedule SET periodic_name='main.cleanup' WHERE periodic_name='main.cleanup_sessions'")
 
