@@ -1,22 +1,29 @@
 # -*- coding: utf-8 -*-
 ##----------------------------------------------------------------------
-## Copyright (C) 2007-2009 The NOC Project
+## Copyright (C) 2007-2010 The NOC Project
 ## See LICENSE for details
 ##----------------------------------------------------------------------
 """
 """
+## Python Modules
 from __future__ import with_statement
+import os
+import datetime
+import re
+import threading
+## Django Modules
 from django.db import models
 from django.contrib.auth.models import User,Group
+from django.core.validators import MaxLengthValidator
+from django.contrib import databrowse
+from django.db.models.signals import class_prepared,pre_save, pre_delete, post_save, post_delete
+## NOC Modules
+from noc import settings
 from noc.lib.fields import BinaryField
 from noc.lib.database_storage import DatabaseStorage as DBS
-import os,datetime,re,datetime,threading
 from noc.main.refbooks.downloaders import downloader_registry
-from django.contrib import databrowse
-from django.db.models.signals import class_prepared,pre_save,pre_delete
-from noc.lib.fields import TextArrayField,PickledField
+from noc.lib.fields import TextArrayField,PickledField,ColorField
 from noc.lib.middleware import get_user
-from noc import settings
 from noc.lib.timepattern import TimePattern as TP
 from noc.lib.timepattern import TimePatternList
 from noc.sa.interfaces.base import interface_registry
@@ -50,7 +57,9 @@ AUDIT_TRAIL_EXCLUDE=set([
     "main_audittrail",
     "kb_kbentryhistory",
     "kb_kbentrypreviewlog",
-    "fm_eventlog"
+    "fm_eventlog",
+    "sa_maptask",
+    "sa_reducetask",
 ])
 ##
 ## Audit trail for INSERT and UPDATE operations
@@ -198,6 +207,55 @@ class Permission(models.Model):
         for p in current-perms:
             Permission.objects.get(name=p).groups.remove(group)
     
+##
+## CSS Styles
+##
+class Style(models.Model):
+    class Meta:
+        verbose_name="Style"
+        verbose_name_plural="Styles"
+    
+    name=models.CharField("Name",max_length=64,unique=True)
+    font_color=ColorField("Font Color",default=0)
+    background_color=ColorField("Background Color",default=0xffffff)
+    bold=models.BooleanField("Bold",default=False)
+    italic=models.BooleanField("Italic",default=False)
+    underlined=models.BooleanField("Underlined",default=False)
+    is_active=models.BooleanField("Is Active",default=True)
+    description=models.TextField("Description",null=True,blank=True)
+    
+    def __unicode__(self):
+        return self.name
+    
+    ##
+    ## CSS class name
+    ##
+    @property
+    def css_class_name(self):
+        return u"noc-color-%d"%self.id
+    
+    ##
+    ##
+    ##
+    @property
+    def style(self):
+        s=u"color: %s; background-color: %s;"%(self.font_color,self.background_color)
+        if self.bold:
+            s+=u" font-weight: bold;"
+        if self.italic:
+            s+=u" font-style: italic;"
+        if self.underlined:
+            s+=u" text-decoration: underline;"
+        return s
+    
+    ##
+    ## CSS style
+    ##
+    @property
+    def css(self):
+        return u".%s { %s }\n"%(self.css_class_name,self.style)
+    
+
 ##
 ## Languages
 ##
@@ -811,3 +869,124 @@ class ChangesQuarantineRule(models.Model):
     description=models.TextField("Description",null=True,blank=True)
     def __unicode__(self):
         return self.name
+##
+## Triggers
+##
+def model_choices():
+    for m in models.get_models():
+        yield (m._meta.db_table,m._meta.db_table)
+    
+class DBTrigger(models.Model):
+    class Meta:
+        verbose_name="Database Trigger"
+        verbose_name_plural="Database Triggers"
+        ordering=("model","order")
+    
+    name=models.CharField("Name",max_length=64,unique=True)
+    model=models.CharField("Model",max_length=128,choices=model_choices())
+    is_active=models.BooleanField("Is Active",default=True)
+    order=models.IntegerField("Order",default=100)
+    description=models.TextField("Description",null=True,blank=True)
+    pre_save_rule=models.ForeignKey(PyRule,verbose_name="Pre-Save Rule",
+                    related_name="dbtrigger_presave_set", limit_choices_to={"interface":"IDBPreSave"},blank=True,null=True)
+    post_save_rule=models.ForeignKey(PyRule,verbose_name="Post-Save Rule",
+                    related_name="dbtrigger_postsave_set", limit_choices_to={"interface":"IDBPostSave"},blank=True,null=True)
+    pre_delete_rule=models.ForeignKey(PyRule,verbose_name="Pre-Delete Rule",
+                    related_name="dbtrigger_predelete_set", limit_choices_to={"interface":"IDBPreDelete"},blank=True,null=True)
+    post_delete_rule=models.ForeignKey(PyRule,verbose_name="Post-Delete Rule",
+                    related_name="dbtrigger_postdelete_set", limit_choices_to={"interface":"IDBPostDelete"},blank=True,null=True)
+    ## State cache
+    _pre_save_triggers={}    # model.meta.db_table -> [rules]
+    _post_save_triggers={}   # model.meta.db_table -> [rules]
+    _pre_delete_triggers={}  # model.meta.db_table -> [rules]
+    _post_delete_triggers={} # model.meta.db_table -> [rules]
+    
+    def __unicode__(self):
+        return "%s: %s"%(self.model,self.name)
+    
+    ##
+    ## Refresh triggers cache
+    ##
+    @classmethod
+    def refresh_cache(cls,*args,**kwargs):
+        # Clear cache
+        cls._pre_save_triggers={}
+        cls._post_save_triggers={}
+        cls._pre_delete_triggers={}
+        cls._post_delete_triggers={}
+        # Add all active triggers
+        for t in cls.objects.filter(is_active=True).order_by("order"):
+            for r in ["pre_save","post_save","pre_delete","post_delete"]:
+                c=getattr(cls,"_%s_triggers"%r)
+                rule=getattr(t,"%s_rule"%r)
+                if rule:
+                    try:
+                        c[t.model]+=[rule]
+                    except KeyError:
+                        c[t.model]=[rule]
+    
+    ##
+    ## Dispatcher for pre-save
+    ##
+    @classmethod
+    def pre_save_dispatch(cls,**kwargs):
+        m=kwargs["sender"]._meta.db_table
+        if m in cls._pre_save_triggers:
+            for t in cls._pre_save_triggers[m]:
+                t(model=kwargs["sender"],instance=kwargs["instance"])
+    
+    ##
+    ## Dispatcher for post-save
+    ##
+    @classmethod
+    def post_save_dispatch(cls,**kwargs):
+        m=kwargs["sender"]._meta.db_table
+        if m in cls._post_save_triggers:
+            for t in cls._post_save_triggers[m]:
+                t(model=kwargs["sender"],instance=kwargs["instance"],created=kwargs["created"])
+    
+    ##
+    ## Dispatcher for pre-delete
+    ##
+    @classmethod
+    def pre_delete_dispatch(cls,**kwargs):
+        m=kwargs["sender"]._meta.db_table
+        if m in cls._pre_delete_triggers:
+            for t in cls._pre_delete_triggers[m]:
+                t(model=kwargs["sender"],instance=kwargs["instance"])
+    
+    ##
+    ## Dispatcher for post-delete
+    ##
+    @classmethod
+    def post_delete_dispatch(cls,**kwargs):
+        m=kwargs["sender"]._meta.db_table
+        if m in cls._post_delete_triggers:
+            for t in cls._post_delete_triggers[m]:
+                t(model=kwargs["sender"],instance=kwargs["instance"])
+    ##
+    ## Called when all models are initialized
+    ##
+    @classmethod
+    def x(cls):
+        self._meta.get_field_by_name("model")[0].choices=[(m._meta.db_table,m._meta.db_table) for m in models.get_models()]
+
+##
+## Install triggers
+##
+if settings.IS_WEB:
+    DBTrigger.refresh_cache() # Load existing triggers
+    # Trigger cache syncronization
+    post_save.connect(DBTrigger.refresh_cache, sender=DBTrigger)
+    post_delete.connect(DBTrigger.refresh_cache, sender=DBTrigger)
+    # Install signal hooks
+    pre_save.connect(DBTrigger.pre_save_dispatch)
+    post_save.connect(DBTrigger.post_save_dispatch)
+    pre_delete.connect(DBTrigger.pre_delete_dispatch)
+    post_delete.connect(DBTrigger.post_delete_dispatch)
+
+##
+## Monkeypatch to change User.username.max_length
+##
+User._meta.get_field("username").max_length=User._meta.get_field("email").max_length
+User._meta.get_field("username").validators=[MaxLengthValidator(User._meta.get_field("username").max_length)]
