@@ -9,6 +9,6 @@ class Migration:
     def forwards(self):
         db.add_column("peer_peer","rpsl_remark",models.CharField("RPSL Remark",max_length=64,null=True,blank=True))
     
-    def backwards(self, orm):
+    def backwards(self):
         db.delete_column("peer_peer","rpsl_remark")
 
