@@ -4,7 +4,6 @@
 ## See LICENSE for details
 ##----------------------------------------------------------------------
 from django.db import models
-from noc.main.menu import Menu
 ##
 ## Time Series
 ##
@@ -28,12 +27,7 @@ class TimeSeries(models.Model):
         c.execute("COMMIT")
     ##
     def as_html(self):
-        return '<div id="nocts_%d" style="width: 600px; height: 400px"></div><script>$("#nocts_%d").nocchart({ajaxURL:"/pm/data/ts/%d/"});</script>'%(self.id,self.id,self.id)
-    ##
-    def view_link(self):
-        return "<a href='/pm/view/ts/%d/'>View</a>"%self.id
-    view_link.short_description="View"
-    view_link.allow_tags=True
+        return '<div id="nocts_%d" style="width: 600px; height: 400px"></div><script>$("#nocts_%d").nocchart({ajaxURL:"/pm/view/ts/%d/data/"});</script>'%(self.id,self.id,self.id)
 ##
 ## Time Series Data
 ##
@@ -47,7 +41,7 @@ class TimeSeriesData(models.Model):
     def __unicode__(self):
         return "%s@%s"%(self.time_series.name,str(self.timestamp))
 ##
-##
+## Chart: the group of timeseries
 ##
 class Chart(models.Model):
     class Meta:
@@ -58,18 +52,4 @@ class Chart(models.Model):
     def __unicode__(self):
         return self.name
     def as_html(self):
-        return '<div id="nocchart_%d" style="width: 600px; height: 400px"></div><script>$("#nocchart_%d").nocchart({ajaxURL:"/pm/data/chart/%d/"});</script>'%(self.id,self.id,self.id)
-
-##
-## Application Menu
-##
-class AppMenu(Menu):
-    app="pm"
-    title="Performance Management"
-    items=[
-        ("Charts", "/pm/", "pm.change_chart"),
-        ("Setup", [
-            ("Time Series", "/admin/pm/timeseries/", "pm.change_timeseries"),
-            ("Charts",      "/admin/pm/chart/",      "pm.change_chart"),
-        ])
-    ]
+        return '<div id="nocchart_%d" style="width: 600px; height: 400px"></div><script>$("#nocchart_%d").nocchart({ajaxURL:"/pm/view/chart/%d/data/"});</script>'%(self.id,self.id,self.id)
