@@ -39,19 +39,22 @@ class Service(SAEService):
                 done(controller,response=c)
             else:
                 e=Error()
-                if script.to_cancel: # Timeout
+                if script.e_timeout:
                     e.code=ERR_TIMEOUT
                     e.text="Timed out"
+                elif script.e_cancel:
+                    e.code=ERR_CANCELLED
+                    e.text="Cancelled"
                 elif script.login_error is not None:
                     e.code=ERR_LOGIN_FAILED
                     e.text=script.login_error
                 else:
                     e.code=ERR_SCRIPT_EXCEPTION
-                    e.text=script.error_traceback
+                    e.text=script.error_traceback if script.error_traceback else "Mysterious death"
                 done(controller,error=e)
         try:
             profile=profile_registry[request.access_profile.profile]
-        except:
+        except KeyError:
             e=Error()
             e.code=ERR_INVALID_PROFILE
             e.text="Invalid profile '%s'"%request.access_profile.profile
@@ -59,7 +62,7 @@ class Service(SAEService):
             return
         try:
             script_class=script_registry[request.script]
-        except:
+        except KeyError:
             e=Error()
             e.code=ERR_INVALID_SCRIPT
             e.text="Invalid script '%s'"%request.script
@@ -185,7 +188,7 @@ class PMCollectorSocket(ListenUDPSocket):
         msg=PMMessage()
         try:
             msg.ParseFromString(data)
-        except:
+        except: # @todo: specify exact type of exception
             return
         # Check hash
         if pmhash(address,self.activator.pm_data_secret,[d.timestamp for d in msg.result]+[d.timestamp for d in msg.data])!=msg.checksum:
@@ -421,18 +424,26 @@ class Activator(Daemon,FSM):
     def run_script(self,script_name,access_profile,callback,**kwargs):
         logging.info("Script %s(%s)"%(script_name,access_profile.address))
         pv,pos,sn=script_name.split(".",2)
-        profile=profile_registry["%s.%s"%(pv,pos)]()        
+        profile=profile_registry["%s.%s"%(pv,pos)]()
         script=script_registry[script_name](profile,self,access_profile,**kwargs)
         with self.script_lock:
             self.script_threads[script]=callback
-            logging.info("%d script threads"%(len(self.script_threads)))
+            logging.info("%d script threads (%d max)"%(len(self.script_threads),self.max_script_threads))
         script.start()
-
+    
     def on_script_exit(self,script):
-        logging.info("Script %s(%s) completed"%(script.name,script.access_profile.address))
+        if script.e_timeout:
+            s="is timed out"
+        elif script.e_cancel:
+            s="is cancelled"
+        elif script.login_error:
+            s="cannot log in"
+        else:
+            s="is completed"
+        logging.info("Script %s(%s) %s"%(script.name,script.access_profile.address,s))
         with self.script_lock:
             cb=self.script_threads.pop(script)
-            logging.info("%d script threads left"%(len(self.script_threads)))
+            logging.info("%d script threads left (%d max)"%(len(self.script_threads),self.max_script_threads))
         cb(script)
         
     def request_call(self,f,*args,**kwargs):
@@ -442,7 +453,28 @@ class Activator(Daemon,FSM):
     ##
     ##
     def ping_check(self,addresses,callback):
-        fping_probe_socket=FPingProbeSocket(self.factory,self.config.get("path","fping"),addresses,callback)
+        def cb(afi,afi_left,result,unreachable):
+            result+=unreachable
+            afi_left.remove(afi)
+            if not afi_left:
+                callback(result)
+        
+        ipv4_addresses=[a for a in addresses if ":" not in a]
+        ipv6_addresses=[a for a in addresses if ":" in a]
+        fping6=self.config.get("path","fping6")
+        
+        result=[]
+        afi_left=set()
+        #
+        if ipv4_addresses:
+            afi_left.add("4")
+        if fping6 and ipv6_addresses:
+            afi_left.add("6")
+        # Launch probes
+        if ipv4_addresses:
+            FPingProbeSocket(self.factory,self.config.get("path","fping"),ipv4_addresses,lambda x: cb("4",afi_left,result,x))
+        if fping6 and ipv6_addresses:
+            FPingProbeSocket(self.factory,fping6,ipv6_addresses,lambda x: cb("6",afi_left,result,x))
     ##
     ##
     ##
@@ -467,7 +499,7 @@ class Activator(Daemon,FSM):
         while not self.script_call_queue.empty():
             try:
                 f,args,kwargs=self.script_call_queue.get_nowait()
-            except:
+            except Queue.Empty:
                 break
             logging.debug("Calling delayed %s(*%s,**%s)"%(f,args,kwargs))
             apply(f,args,kwargs)
@@ -695,7 +727,7 @@ class Activator(Daemon,FSM):
             try:
                 logging.debug("Adding ignore rule: %s | %s"%(r.left_re,r.right_re))
                 self.ignore_event_rules+=[(re.compile(r.left_re,re.IGNORECASE),re.compile(r.right_re,re.IGNORECASE))]
-            except Exception,why:
+            except re.error,why:
                 logging.error("Failed to compile ignore event rule: %s,%s. skipping"%(l,r))
     ##
     ## Cancel stale scripts
@@ -705,10 +737,10 @@ class Activator(Daemon,FSM):
         to_cancel=[st for st in self.script_threads.keys() if st.is_stale()]
         for script in to_cancel:
             logging.info("Canceling stale script %s(%s)"%(script.name,script.access_profile.address))
-            st.cancel_script()
-        
+            script.cancel_script()
+    
     # Signal handlers
-
+    
     # SIGUSR1 returns process info
     def SIGUSR1(self,signo,frame):
         s=[
@@ -724,7 +756,7 @@ class Activator(Daemon,FSM):
         while True:
             try:
                 pid,status=os.waitpid(-1,os.WNOHANG)
-            except:
+            except OSError:
                 break
             if pid:
                 logging.debug("Zombie pid=%d is hunted and mercilessly killed"%pid)
