@@ -5,16 +5,21 @@
 ## Copyright (C) 2007-2010 The NOC Project
 ## See LICENSE for details
 ##----------------------------------------------------------------------
+
+## Python modules
+import pprint
+import os
+## Django modules
 from django.contrib import admin
 from django.shortcuts import get_object_or_404
 from django import forms
 from django.db.models import Q
 from django.contrib.auth.models import User,Group
-from noc.lib.app import ModelApplication,site,Permit,PermitSuperuser,HasPerm,PermissionDenied
+## NOC modules
+from noc.lib.app import ModelApplication, site, Permit, PermitSuperuser, HasPerm, PermissionDenied, view
 from noc.sa.models import *
 from noc.settings import config
 from noc.lib.fileutils import in_dir
-import pprint,types,socket,csv,os
 ##
 ## Validating form for managed object
 ##
@@ -42,18 +47,21 @@ class ManagedObjectAdminForm(forms.ModelForm):
 def action_links(obj):
     r=[]
     try:
-        r+=[("Config","cm:config:view",[obj.config.id])]
+        r+=[("Config", "cm:config:view", [obj.config.id])]
     except:
         pass
     try:
         obj.profile
-        r+=[("Scripts","sa:managedobject:scripts",[obj.id])]
+        r+=[("Scripts", "sa:managedobject:scripts", [obj.id])]
     except:
         pass
-    r+=[("Addresses","sa:managedobject:addresses",[obj.id])]
-    return "<br/>".join(["<a href='%s'>%s</a>"%(site.reverse(view,*params),title) for title,view,params in r])
+    r+=[("Addresses", "sa:managedobject:addresses", [obj.id])]
+    r+=[("Attributes", "sa:managedobject:attributes", [obj.id])]
+    s=["<select onchange='document.location=this.options[this.selectedIndex].value;'>","<option>---</option>"]+["<option value='%s'>%s</option>"%(site.reverse(view,*params),title) for title, view, params in r]+["</select>"]
+    return "".join(s)
 action_links.short_description="Actions"
 action_links.allow_tags=True
+
 ##
 ## Display object status
 ##
@@ -72,13 +80,30 @@ object_status.allow_tags=True
 def script_reduce(task):
     mt=task.maptask_set.all()[0]
     if mt.status!="C":
-        return "Task failed: "+str(mt.script_result)
+        return "Task failed: "+str(mt.script_result["text"])
     return mt.script_result
+
+##
+## Attributes inline form
+##
+class ManagedObjectAttributeInlineForm(forms.ModelForm):
+    class Meta:
+        model=ManagedObjectAttribute
+    
+##
+## Attributes inline
+##
+class ManagedObjectAttributeInline(admin.TabularInline):
+    form=ManagedObjectAttributeInlineForm
+    model=ManagedObjectAttribute
+    extra=3
+
 ##
 ## ManagedObject admin
 ##
 class ManagedObjectAdmin(admin.ModelAdmin):
     form=ManagedObjectAdminForm
+    inlines=[ManagedObjectAttributeInline]
     fieldsets=(
         (None,{
             "fields": ("name","is_managed","administrative_domain","activator","profile_name","description")
@@ -214,7 +239,7 @@ class ManagedObjectApplication(ModelApplication):
         try:
             result=task.get_result(block=False)
         except ReduceTask.NotReady:
-            return self.render_wait(request,subject="Script %s"%script,text="Processing scirpt. Please wait ...")
+            return self.render_wait(request,subject="Script %s"%script,text="Processing script. Please wait ...")
         # Format result
         if isinstance(result,basestring):
             pass # Do not convert strings
@@ -254,10 +279,21 @@ class ManagedObjectApplication(ModelApplication):
     ##
     def view_addresses(self,request,object_id):
         o=get_object_or_404(ManagedObject,id=int(object_id))
-        return self.render(request,"addresses.html",{"addresses":o.ipv4address_set.all(),"object":o})
+        return self.render(request,"addresses.html",{"addresses":o.address_set.order_by("address"),"object":o})
     view_addresses.url=r"(?P<object_id>\d+)/addresses/"
     view_addresses.url_name="addresses"
     view_addresses.access=HasPerm("change")
+    
+    ##
+    ## Display all attributes
+    ##
+    @view(  url=r"(?P<object_id>\d+)/attributes/",
+            url_name="attributes",
+            access=HasPerm("change"))
+    def view_attributes(self,request,object_id):
+        o=get_object_or_404(ManagedObject,id=int(object_id))
+        return self.render(request,"attributes.html",{"attributes":o.managedobjectattribute_set.order_by("key"),"object":o})
+    
     ##
     def user_access_list(self,user):
         return [s.selector.name for s in UserAccess.objects.filter(user=user)]
