@@ -18,9 +18,12 @@ else:
     ## Pure-python replacement for Python 2.5
     ##
     def combinations(s,n=None):
-        ss=s[:]
+        try:
+            ss=s.copy()
+        except AttributeError:
+            ss=s[:]
         while ss:
-            c=ss.pop(0)
+            c=ss.pop()
             for cc in ss:
                 yield (c,cc)
     
@@ -99,7 +102,8 @@ class FIB(object):
         if not s:
             print "FIB: Empty"
             return
-        print "FIB size",s,"of",ms,"(%5.2f%%)"%(float(s*100)/float(ms))
+        if ms:
+            print "FIB size",s,"of",ms,"(%5.2f%%)"%(float(s*100)/float(ms))
         n_interfaces=0
         for i in self.interfaces.values():
             n_interfaces+=len(i)
