@@ -7,8 +7,14 @@
 ##----------------------------------------------------------------------
 """
 """
-import re,types,datetime
+## Python modukes
+import re
+import types
+import datetime
+## NOC Modules
 from noc.lib.text import list_to_ranges,ranges_to_list,list_to_ranges
+from noc.lib.ip import IPv6
+from noc.lib.validators import *
 
 try:
     from django import forms
@@ -43,7 +49,7 @@ class Parameter(object):
     ## Perform input parameter normalization
     ##
     def clean(self,value):
-        self.raise_error(value)
+        return value
     ##
     ## Clean up script input parameters.
     ## Call .clean() by default
@@ -75,31 +81,31 @@ class Parameter(object):
 ##
 class ORParameter(Parameter):
     """
-    >>> ORParameter(IntParameter(),IPParameter()).clean(10)
+    >>> ORParameter(IntParameter(),IPv4Parameter()).clean(10)
     10
-    >>> ORParameter(IntParameter(),IPParameter()).clean("192.168.1.1")
+    >>> ORParameter(IntParameter(),IPv4Parameter()).clean("192.168.1.1")
     '192.168.1.1'
-    >>> ORParameter(IntParameter(),IPParameter()).clean("xxx") #doctest: +IGNORE_EXCEPTION_DETAIL
+    >>> ORParameter(IntParameter(),IPv4Parameter()).clean("xxx") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    InterfaceTypeError: IPParameter: 'xxx'
-    >>> (IntParameter()|IPParameter()).clean(10)
+    InterfaceTypeError: IPv4Parameter: 'xxx'
+    >>> (IntParameter()|IPv4Parameter()).clean(10)
     10
-    >>> (IntParameter()|IPParameter()).clean("192.168.1.1")
+    >>> (IntParameter()|IPv4Parameter()).clean("192.168.1.1")
     '192.168.1.1'
-    >>> (IntParameter()|IPParameter()).clean("xxx") #doctest: +IGNORE_EXCEPTION_DETAIL
+    >>> (IntParameter()|IPv4Parameter()).clean("xxx") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    InterfaceTypeError: IPParameter: 'xxx'
-    >>> (IntParameter()|IPParameter()).clean(None) #doctest: +IGNORE_EXCEPTION_DETAIL
+    InterfaceTypeError: IPv4Parameter: 'xxx'
+    >>> (IntParameter()|IPv4Parameter()).clean(None) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    InterfaceTypeError: IPParameter: None.
-    >>> (IntParameter(required=False)|IPParameter(required=False)).clean(None)
-    >>> (IntParameter(required=False)|IPParameter()).clean(None) #doctest: +IGNORE_EXCEPTION_DETAIL
+    InterfaceTypeError: IPv4Parameter: None.
+    >>> (IntParameter(required=False)|IPv4Parameter(required=False)).clean(None)
+    >>> (IntParameter(required=False)|IPv4Parameter()).clean(None) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    InterfaceTypeError: IPParameter: None.
+    InterfaceTypeError: IPv4Parameter: None.
     """
     def __init__(self,left,right):
         super(ORParameter,self).__init__()
@@ -506,7 +512,10 @@ class DictParameter(Parameter):
         out_value={}
         for a_name,attr in self.attrs.items():
             if a_name not in in_value and attr.required:
-                self.raise_error(value,"Attribute '%s' is required"%a_name)
+                if attr.default:
+                    out_value[a_name]=attr.default
+                else:
+                    self.raise_error(value,"Attribute '%s' is required in %s"%(a_name,in_value))
             if a_name in in_value:
                 try:
                     out_value[a_name]=attr.clean(in_value[a_name])
@@ -554,8 +563,8 @@ class DictParameter(Parameter):
             if a_name in in_value:
                 try:
                     out_value[a_name]=attr.script_clean_result(profile,in_value[a_name])
-                except InterfaceTypeError:
-                    self.raise_error(value,"Invalid value for '%s'"%a_name)
+                except InterfaceTypeError,why:
+                    self.raise_error(value,"Invalid value for '%s': %s"%(a_name,str(why)))
                 del in_value[a_name]
         for k,v in in_value.items():
             out_value[k]=v
@@ -579,19 +588,19 @@ class DateTimeParameter(StringParameter):
 ##
 ##
 ##
-class IPParameter(StringParameter):
+class IPv4Parameter(StringParameter):
     """
-    >>> IPParameter().clean("192.168.0.1")
+    >>> IPv4Parameter().clean("192.168.0.1")
     '192.168.0.1'
-    >>> IPParameter().clean("192.168.0.256") #doctest: +IGNORE_EXCEPTION_DETAIL
+    >>> IPv4Parameter().clean("192.168.0.256") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    InterfaceTypeError: IPParameter: '192.168.0.256'
+    InterfaceTypeError: IPvParameter: '192.168.0.256'
     """
     def clean(self,value):
         if value is None and self.default is not None:
             return self.default
-        v=super(IPParameter,self).clean(value)
+        v=super(IPv4Parameter,self).clean(value)
         X=v.split(".")
         if len(X)!=4:
             self.raise_error(value)
@@ -608,18 +617,18 @@ class IPv4PrefixParameter(StringParameter):
     """
     >>> IPv4PrefixParameter().clean("192.168.0.0/16")
     '192.168.0.0/16'
-    >>> IPParameter().clean("192.168.0.256") #doctest: +IGNORE_EXCEPTION_DETAIL
+    >>> IPv4PrefixParameter().clean("192.168.0.256") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    InterfaceTypeError: IPParameter: '192.168.0.256'
-    >>> IPParameter().clean("192.168.0.0/33") #doctest: +IGNORE_EXCEPTION_DETAIL
+    InterfaceTypeError: IPv4PrefixParameter: '192.168.0.256'
+    >>> IPv4PrefixParameter().clean("192.168.0.0/33") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    InterfaceTypeError: IPParameter: '192.168.0.0/33'
-    >>> IPParameter().clean("192.168.0.0/-5") #doctest: +IGNORE_EXCEPTION_DETAIL
+    InterfaceTypeError: IPv4PrefixParameter: '192.168.0.0/33'
+    >>> IPv4PrefixParameter().clean("192.168.0.0/-5") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    InterfaceTypeError: IPParameter: '192.168.0.0/-5'
+    InterfaceTypeError: IPv4PrefixParameter: '192.168.0.0/-5'
     """
     def clean(self,value):
         if value is None and self.default is not None:
@@ -644,6 +653,98 @@ class IPv4PrefixParameter(StringParameter):
             self.raise_error(value)
         return v
     
+
+##
+## IPv6 Parameter
+##
+class IPv6Parameter(StringParameter):
+    """
+    >>> IPv6Parameter().clean("::")
+    '::'
+    >>> IPv6Parameter().clean("::1")
+    '::1'
+    >>> IPv6Parameter().clean("2001:db8::1")
+    '2001:db8::1'
+    >>> IPv6Parameter().clean("2001:db8::")
+    '2001:db8::'
+    >>> IPv6Parameter().clean("::ffff:192.168.0.1")
+    '::ffff:192.168.0.1'
+    >>> IPv6Parameter().clean('g::')  #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: IPv6Parameter: 'g::'.
+    >>> IPv6Parameter().clean("0:00:0:0:0::1")
+    '::1'
+    >>> IPv6Parameter().clean("::ffff:c0a8:1")
+    '::ffff:192.168.0.1'
+    >>> IPv6Parameter().clean("2001:db8:0:7:0:0:0:1")
+    '2001:db8:0:7::1'
+    """
+    def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
+        v=super(IPv6Parameter,self).clean(value)
+        if not is_ipv6(v):
+            self.raise_error(value)
+        return IPv6(v).normalized.address
+    
+
+##
+##
+##
+class IPv6PrefixParameter(StringParameter):
+    """
+    >>> IPv6PrefixParameter().clean("::/128")
+    '::/128'
+    >>> IPv6PrefixParameter().clean("2001:db8::/32")
+    '2001:db8::/32'
+    >>> IPv6PrefixParameter().clean("2001:db8::/129") #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: IPv6PrefixParameter: '2001:db8::/129'
+    >>> IPv6PrefixParameter().clean("2001:db8::/g") #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: IPv6PrefixParameter: '2001:db8::/g'
+    >>> IPv6PrefixParameter().clean("2001:db8::") #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: IPv6PrefixParameter: '2001:db8::'
+    """
+    def clean(self, value):
+        if value is None and self.default is not None:
+            return self.default
+        v=super(IPv6PrefixParameter,self).clean(value)
+        if "/" not in v:
+            self.raise_error(value)
+        n,m=v.split("/",1)
+        try:
+            m=int(m)
+        except:
+            self.raise_error(value)
+        if m<0 or m>128:
+            self.raise_error(value)
+        n=IPv6Parameter().clean(n)
+        return "%s/%d"%(n,m)
+    
+
+##
+## IPv4/IPv6 parameter
+##
+class IPParameter(StringParameter):
+    def clean(self,value):
+        """
+        >>> IPParameter().clean("192.168.0.1")
+        '192.168.0.1'
+        >>> IPParameter().clean("::")
+        '::'
+        """
+        if ":" in value:
+            return IPv6Parameter().clean(value)
+        else:
+            return IPv4Parameter().clean(value)
+    
+
 ##
 ##
 ##
