@@ -19,3 +19,9 @@ class Profile(noc.sa.profiles.Profile):
     pattern_prompt=r"^[<#]\S+?[>#]"
     command_more=" "
     config_volatile=["^%.*?$"]
+
+    def generate_prefix_list(self,name,pl,strict=True):
+        p="ip ip-prefix %s permit %%s"%name
+        if not strict:
+            p+=" le 32"
+        return "undo ip ip-prefix %s\n"%name+"\n".join([p%x.replace("/"," ") for x in pl])
