@@ -1,15 +1,20 @@
 # -*- coding: utf-8 -*-
 ##----------------------------------------------------------------------
-## Copyright (C) 2007-2009 The NOC Project
+## Profile base class
+##----------------------------------------------------------------------
+## Copyright (C) 2007-2010 The NOC Project
 ## See LICENSE for details
 ##----------------------------------------------------------------------
 """
 """
-from noc.lib.ip import bits_to_netmask
+# Python Modules
+import re
+# NOC Modules
 from noc.lib.registry import Registry
 from noc.lib.ecma48 import strip_control_sequences
 from noc.sa.interfaces import InterfaceTypeError
-import re
+from noc.lib.ip import *
+
 ##
 ##
 ##
@@ -101,13 +106,12 @@ class Profile(object):
     # or netmask should be converted to traditional formats
     requires_netmask_conversion=False
     #
-    # Converts ipv4 prefix to the format acceptable by router
+    # Converts ip prefix to the format acceptable by router
     #
     def convert_prefix(self,prefix):
         if "/" in prefix and self.requires_netmask_conversion:
-            net,mask=prefix.split("/")
-            mask=bits_to_netmask(mask)
-            return "%s %s"%(net,mask)
+            p=IPv4(prefix)
+            return "%s %s"%(prefix.address,prefix.netmask.address)
         return prefix
     #
     # Leave 00:11:22:33:44:55 style MAC-address untouched
@@ -192,6 +196,34 @@ class Profile(object):
                 rx=re.compile(r,re.DOTALL|re.MULTILINE)
                 cfg=rx.sub("",cfg)
         return unicode(cfg,"utf8","ignore").encode("utf8") # Prevent serialization errors
+    
+    #
+    # Compare two versions.
+    # Must return:
+    #    <0 , if v1<v2
+    #     0 , if v1==v2
+    #    >0 , if v1>v2
+    #  None , if v1 and v2 cannot be compared
+    #
+    # Default implementation compares a versions in format
+    # N1. .. .NM
+    #
+    @classmethod
+    def cmp_version(self,v1,v2):
+        n1=[int(x) for x in v1.split(".")]
+        n2=[int(x) for x in v2.split(".")]
+        l1=len(n1)
+        l2=len(n2)
+        if l1>l2:
+            n2+=[None]*(l1-l2)
+        elif l1<l2:
+            n1+=[None]*(l2-l1)
+        for c1,c2 in zip(n1,n2):
+            r=cmp(c1,c2)
+            if r!=0:
+                return r
+        return 0
+    
     #
     # Highligh config
     # Try to include profile's highlight module and use its ConfigLexer
