@@ -450,7 +450,7 @@ class SAE(Daemon):
     def script(self,object,script_name,callback,**kwargs):
         def script_callback(transaction,response=None,error=None):
             if error:
-                logging.error("script(%s,**%s) failed: %s"%(script_name,kwargs,error.text))
+                logging.error("script(%s,%s,**%s) failed: %s"%(script_name,object,kwargs,error.text))
                 callback(error=error)
                 return
             result=response.result
@@ -487,6 +487,11 @@ class SAE(Daemon):
             r.access_profile.snmp_ro       = object.snmp_ro
         if object.snmp_rw:
             r.access_profile.snmp_rw       = object.snmp_rw
+        attrs=[(a.key,a.value) for a in object.managedobjectattribute_set.all()]
+        for k,v in attrs:
+            a=r.access_profile.attrs.add()
+            a.key=str(k)
+            a.value=v
         for k,v in kwargs.items():
             a=r.kwargs.add()
             a.key=str(k)
@@ -573,7 +578,7 @@ class SAE(Daemon):
                         mt.save()
                         return
                 mt.status="F"
-                mt.script_result=error.text
+                mt.script_result=dict(code=error.code,text=error.text)
             else:
                 mt.status="C"
                 mt.script_result=result
@@ -590,6 +595,7 @@ class SAE(Daemon):
         for mt in MapTask.objects.filter(status="W",next_try__lte=t):
             if mt.task.stop_time<t: # Task timeout
                 mt.status="F"
+                mt.script_result=dict(code=ERR_TIMEOUT, text="Timed out")
                 mt.save()
                 continue
             mt.status="R"
