@@ -5,17 +5,24 @@
 ##----------------------------------------------------------------------
 """
 """
+## Python modules
 from __future__ import with_statement
-from noc.lib.fsm import StreamFSM
-from noc.lib.registry import Registry
-from noc.lib.nbsocket import PTYSocket,UDPSocket,SocketTimeoutError
-from noc.lib.debug import format_frames,get_traceback_frames
-from noc.lib.text import replace_re_group
-from noc.sa.protocols.sae_pb2 import TELNET,SSH,HTTP
-from noc.sa.profiles import profile_registry
-from noc.settings import config
-import logging,re,threading,Queue,urllib,httplib,random,base64,hashlib,cPickle,sys,time,datetime
-
+import sys
+import time
+import datetime
+import logging
+import re
+import threading
+import thread
+import Queue
+import urllib
+import httplib
+import random
+import base64
+import hashlib
+import cPickle
+import ctypes
+## Third-party modules
 try:
     from pysnmp.carrier.asynsock.dispatch import AsynsockDispatcher
     from pysnmp.carrier.asynsock.dgram import udp
@@ -25,6 +32,15 @@ try:
 except ImportError:
     HAS_SNMP=False
 
+## NOC Modules
+from noc.lib.fsm import StreamFSM
+from noc.lib.registry import Registry
+from noc.lib.nbsocket import PTYSocket,UDPSocket,SocketTimeoutError
+from noc.lib.debug import format_frames,get_traceback_frames
+from noc.lib.text import replace_re_group
+from noc.sa.protocols.sae_pb2 import TELNET,SSH,HTTP
+from noc.sa.profiles import profile_registry
+from noc.settings import config
 
 ##
 ##
@@ -34,10 +50,17 @@ scheme_id={
     "ssh"    : SSH,
     "http"   : HTTP,
 }
-##
+## Operation timed out
 class TimeOutError(Exception): pass
+## Login Failed
 class LoginError(Exception): pass
+## CLI reports syntax error
 class CLISyntaxError(Exception): pass
+## Feature is not supported on current platform/software/feature set
+class NotSupportedError(Exception): pass
+## Commands returns result that cannot be parsed
+class UnexpectedResultError(Exception): pass
+
 ##
 ##
 ##
@@ -92,9 +115,13 @@ script_registry=ScriptRegistry()
 ##
 ##
 ##
+_execute_chain=[]
 class ScriptBase(type):
     def __new__(cls,name,bases,attrs):
+        global _execute_chain
         m=type.__new__(cls,name,bases,attrs)
+        m._execute_chain=_execute_chain
+        _execute_chain=[]
         m.implements=[c() for c in m.implements]
         script_registry.register(m.name,m)
         if m.name and not m.name.startswith("Generic."):
@@ -114,6 +141,35 @@ class ConfigurationContextManager(object):
             self.script.leave_config()
         else:
             raise exc_type, exc_val
+##
+## Mark cancelable part of script
+##
+class CancellableContextManager(object):
+    def __init__(self,script):
+        self.script=script
+    
+    def __enter__(self):
+        self.script.is_cancelable=True
+    
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.is_cancelable=False
+    
+
+##
+## Silently ignore specific exceptions
+##
+class IgnoredExceptionsContextManager(object):
+    def __init__(self, iterable):
+        self.exceptions=set(iterable)
+    
+    def __enter__(self):
+        pass
+    
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if exc_type and exc_type in self.exceptions:
+            return True # Supress exception
+    
+
 ##
 ##
 ##
@@ -121,6 +177,10 @@ class Script(threading.Thread):
     __metaclass__=ScriptBase
     name=None
     description=None
+    # Enable call cache
+    # If True, script result will be cached and reused
+    # during lifetime of parent script
+    cache=False
     # Interfaces list. Each element of list must be Interface subclass
     implements=[]
     # Scripts required by generic script.
@@ -135,12 +195,20 @@ class Script(threading.Thread):
     #
     LoginError=LoginError
     CLISyntaxError=CLISyntaxError
-
+    NotSupportedError=NotSupportedError
+    UnexpectedResultError=UnexpectedResultError
+    #
+    _execute_chain=[]
+    
     def __init__(self,profile,activator,access_profile,parent=None,**kwargs):
         self.start_time=time.time()
-        self.to_cancel=False
         self.parent=parent
         self.access_profile=access_profile
+        self.attrs={}
+        for a in access_profile.attrs:
+            self.attrs[a.key]=a.value
+        import logging
+        logging.debug(repr(self.attrs))
         if self.access_profile.address:
             p=self.access_profile.address
         elif self.access_profile.path:
@@ -160,6 +228,7 @@ class Script(threading.Thread):
             self.snmp=None
         self.status=False
         self.result=None
+        self.call_cache={} # Suitable only when self.parent is None. Cached results for scripts marked with "cache"
         self.error_traceback=None
         self.login_error=None
         self.strip_echo=True
@@ -168,6 +237,11 @@ class Script(threading.Thread):
         self.need_to_save=False
         self.to_disable_pager=not self.parent and self.profile.command_disable_pager
         self.log_cli_sessions_path=None # Path to log CLI session
+        self.is_cancelable=False # Can script be cancelled
+        self.e_timeout=False # Script terminated with timeout
+        self.e_cancel=False # Scrcipt cancelled
+        self._thread_id=None # Python 2.5 compatibility
+        
         if self.parent:
             self.log_cli_sessions_path=self.parent.log_cli_sessions_path
         elif self.activator.log_cli_sessions\
@@ -180,6 +254,75 @@ class Script(threading.Thread):
                 ("ts",datetime.datetime.now().strftime("%Y-%m-%d-%H-%M-%S"))]:
                 self.log_cli_sessions_path=self.log_cli_sessions_path.replace("{{%s}}"%k,v)
             self.cli_debug("IP: %s SCRIPT: %s"%(self.access_profile.address,self.name),"!")
+    
+    ##
+    ## execute method decorator
+    ##
+    @classmethod
+    def match(cls, *args, **kwargs):
+        def decorate(f):
+            global _execute_chain
+            # Append to the execute chain
+            _execute_chain+=[(x, f)]
+            return f
+        
+        # Compile check function
+        c=[]
+        if args:
+            c+=args
+        for k,v in kwargs.items():
+            # Split to field name and lookup operator
+            if "__" in k:
+                f,o=k.split("__")
+            else:
+                f=k
+                o="exact"
+            # Check field name
+            if f not in ("vendor", "platform", "version", "image"):
+                raise Exception("Invalid field '%s'"%f)
+            # Compile lookup functions
+            if o=="exact":
+                c+=[lambda x,f=f,v=v: x[f]==v]
+            elif o=="iexact":
+                c+=[lambda x,f=f,v=v: x[f].lower()==v.lower()]
+            elif o=="startswith":
+                c+=[lambda x,f=f,v=v: x[f].startswith(v)]
+            elif o=="istartswith":
+                c+=[lambda x,f=f,v=v: x[f].lower().startswith(v.lower())]
+            elif o=="endswith":
+                c+=[lambda x,f=f,v=v: x[f].endswith(v)]
+            elif o=="iendswith":
+                c+=[lambda x,f=f,v=v: x[f].lower().endswith(v.lower())]
+            elif o=="contains":
+                c+=[lambda x,f=f,v=v: v in x[f]]
+            elif o=="icontains":
+                c+=[lambda x,f=f,v=v: v.lower() in x[f].lower()]
+            elif o=="in":
+                c+=[lambda x,f=f,v=v: x[f] in v]
+            elif o=="regex":
+                c+=[lambda x,f=f,v=re.compile(v): v.search(x[f]) is not None]
+            elif o=="iregex":
+                c+=[lambda x,f=f,v=re.compile(v, re.IGNORECASE): v.search(x[f]) is not None]
+            elif o=="isempty": # Empty string or null
+                c+=[lambda x,f=f,v=v: not x[f] if v else x[f]]
+            elif f=="version":
+                if o=="lt": # <
+                    c+=[lambda x,f=v,v=v,p=self.profile: p.cmp_version(x[v],v)<0 ]
+                elif o=="lte": # <=
+                    c+=[lambda x,f=v,v=v,p=self.profile: p.cmp_version(x[v],v)<=0 ]
+                elif o=="gt": # >
+                    c+=[lambda x,f=v,v=v,p=self.profile: p.cmp_version(x[v],v)>0 ]
+                elif o=="gte": # >=
+                    c+=[lambda x,f=v,v=v,p=self.profile: p.cmp_version(x[v],v)>=0 ]
+                else:
+                    raise Exception("Invalid lookup operation: %s"%o)
+            else:
+                raise Exception("Invalid lookup operation: %s"%o)
+        # Combine expressions into single lambda
+        x=reduce(lambda x,y: lambda v,x=x,y=y: x(v) and y(v), c, lambda x: True)
+        # Return decorated function
+        return decorate
+    
     ##
     ##
     ##
@@ -211,17 +354,55 @@ class Script(threading.Thread):
     
     def error(self,msg):
         logging.error("[%s] %s"%(self.debug_name,msg))
-        
+    
+    ##
+    ## Return root script
+    ##
+    @property
+    def root(self):
+        if self.parent:
+            return self.parent.root
+        else:
+            return self
+    
+    ##
+    ## Get cached result or raise KeyError
+    ##
+    def get_cache(self,key1,key2):
+        s=self.root
+        return s.call_cache[repr(key1)][repr(key2)]
+    
+    ##
+    ## Set cacked result
+    ##
+    def set_cache(self,key1,key2,value):
+        key1=repr(key1)
+        key2=repr(key2)
+        s=self.root
+        if key1 not in self.call_cache:
+            self.call_cache[key1]={}
+        self.call_cache[key2]=value
+    
     def guarded_run(self):
         self.debug("Guarded run")
         # Enforce interface type checking
         for i in self.implements:
             self.kwargs=i.script_clean_input(self.profile,**self.kwargs)
+        # Use cached result when available
+        if self.cache and self.parent is not None:
+            try:
+                return self.get_cache(self.name,self.kwargs)
+            except KeyError:
+                pass
         # Calling script body
+        self._thread_id=thread.get_ident()
         result=self.execute(**self.kwargs)
         # Enforce interface result checking
         for i in self.implements:
             result=i.script_clean_result(self.profile,result)
+        # Cache result when required
+        if self.cache and self.parent is not None:
+            self.set_cache(self.name,self.kwargs,result)
         self.debug("Script returns with result: %s"%result)
         return result
         
@@ -232,13 +413,18 @@ class Script(threading.Thread):
         self.debug("Running")
         result=None
         try:
-            result=self.guarded_run()
+            with self.cancelable():
+                result=self.guarded_run()
             self.result=self.serialize_result(result)
             if self.parent is None and self.need_to_save and self.profile.command_save_config:
                 self.debug("Saving config")
                 self.cli(self.profile.command_save_config)
         except TimeOutError:
             self.error("Timed out")
+            self.e_timeout=True
+        except CancelledError:
+            self.error("Cancelled")
+            self.e_cancel=True
         except self.LoginError,why:
             self.login_error=why.args[0]
             self.error("Login failed: %s"%self.login_error)
@@ -256,9 +442,22 @@ class Script(threading.Thread):
         if self.snmp:
             self.snmp.close()
         self.activator.on_script_exit(self)
-        
+    
+    ##
+    ## Default script behavior:
+    ## Pass through _execute_chain and call appropriative handler
+    ##
     def execute(self,**kwargs):
-        return None
+        if self._execute_chain and not self.name.endswith(".get_version"):
+            # Get version information
+            v=self.scripts.get_version()
+            # Find and execute proper handler
+            for c,f in self._execute_chain:
+                if c(v):
+                    return f(self, **kwargs)
+            # Raise error 
+            raise NotSupportedError()
+    
     ##
     ## Request CLI provider's queue
     ## Handle cancel condition
@@ -268,11 +467,12 @@ class Script(threading.Thread):
             try:
                 return self.cli_provider.queue.get(block=True,timeout=1)
             except Queue.Empty:
-                if self.to_cancel:
-                    self.error("Canceled")
-                    raise TimeOutError()
-                else:
-                    continue
+                pass
+            except thread.error:
+                # Bug in python's Queue module:
+                # Sometimes, tries to release unacquired lock
+                time.sleep(1)
+        
     
     def request_cli_provider(self):
         if self.parent:
@@ -383,12 +583,52 @@ class Script(threading.Thread):
     ## Cancel script
     ##
     def cancel_script(self):
-        self.to_cancel=True
-        
+        # Can cancel only inside guarded_run
+        if not self.is_cancelable:
+            self.error("Cannot cancel non-cancelable scipt")
+            return
+        if not self.isAlive():
+            self.error("Trying to kill already dead thread")
+            return self.activator.on_script_exit(self)
+        if not self._thread_id:
+            self.error("Cannot cancel the script without thread_id")
+            return
+        # Raise CancelledError in script's thread
+        r=ctypes.pythonapi.PyThreadState_SetAsyncExc(ctypes.c_long(self._thread_id), ctypes.py_object(CancelledError))
+        if r==1:
+            self.debug("Cancel event sent")
+            # Remote exception raised.
+            if self.cli_provider:
+                # Awake script thread if waiting for CLI
+                self.cli_provider.queue.put(None)
+                self.cli_provider.queue.put(None)
+        elif r>1:
+            # Failed to raise exception
+            # Revert back thread state
+            ctypes.pythonapi.PyThreadState_SetAsyncExc(ctypes.c_long(self.ident), None)
+            self.error("Failed to cancel script")
+    
+    ##
+    ## Debugging helper to hang the script
+    ##
+    def hang(self):
+        logging.debug("Hanging script")
+        e=threading.Event()
+        while True:
+            e.wait(1)
+    
     ## Returns configuration context
     def configure(self):
         return ConfigurationContextManager(self)
-        
+    
+    ## Return cancelable context
+    def cancelable(self):
+        return CancellableContextManager(self)
+    
+    ## Return ignorable exceptions context
+    def ignored_exceptions(self, iterable):
+        return IgnoredExceptionsContextManager(iterable)
+    
     # Enter configuration mote
     def enter_config(self):
         if self.profile.command_enter_config:
@@ -418,11 +658,51 @@ class Script(threading.Thread):
             self.request_cli_provider()
         return self.cli_provider.motd
     motd=property(motd)
+    
+    ##
+    ## Match s against regular expression rx using re.search
+    ## Raise UnexpectedResultError if regular expression is not matched.
+    ## Returns match object.
+    ## rx can be string or compiled regular expression
+    ##
+    def re_search(self, rx, s, flags=0):
+        if isinstance(rx,basestring):
+            rx=re.compile(rx,flags)
+        match=rx.search(s)
+        if match is None:
+            raise self.UnexpectedResultError()
+        return match
+    
+    ##
+    ## Match s against regular expression rx using re.match
+    ## Raise UnexpectedResultError if regular expression is not matched.
+    ## Returns match object.
+    ## rx can be string or compiled regular expression
+    ##
+    def re_match(self, rx, s, flags=0):
+        if isinstance(rx,basestring):
+            rx=re.compile(rx,flags)
+        match=rx.match(s)
+        if match is None:
+            raise self.UnexpectedResultError()
+        return match
+    
+    ##
+    ## Find first matching regular expression
+    ## or raise Unexpected result error
+    ##
+    def find_re(self, iter, s):
+        for r in iter:
+            if r.search(s):
+                return r
+        raise self.UnexpectedResultError()
+    
 
 ##
 ##
 ##
 class TimeOutError(Exception): pass
+class CancelledError(Exception): pass
 ##
 ##
 ##
@@ -489,7 +769,7 @@ class CLI(StreamFSM):
             self.more_patterns=[x[0] for x in self.profile.pattern_more]
             self.more_commands=[x[1] for x in self.profile.pattern_more]
         self.pager_patterns="|".join([r"(%s)"%p for p in self.more_patterns])
-        StreamFSM.__init__(self)
+        super(CLI,self).__init__(async_throttle=32)
     
     def on_read(self,data):
         self.debug("on_read: %s"%repr(data))
@@ -636,6 +916,11 @@ class CLITelnetSocket(ScriptSocket,CLI,PTYSocket):
         CLI.__init__(self,profile,access_profile)
         PTYSocket.__init__(self,factory,cmd_args)
         ScriptSocket.__init__(self)
+    
+    def is_stale(self):
+        self.async_check_fsm()
+        return PTYSocket.is_stale(self)
+
 ##
 ##
 ##
@@ -643,13 +928,21 @@ class CLISSHSocket(ScriptSocket,CLI,PTYSocket):
     TTL=30
     def __init__(self,factory,profile,access_profile):
         logging.debug("CLISSHSocket connecting '%s'"%access_profile.address)
-        cmd_args=[config.get("path","ssh"),"-o","StrictHostKeyChecking=no","-o","UserKnownHostsFile=/dev/null","-l",access_profile.user]
+        cmd_args=[config.get("path","ssh"),
+            "-o","StrictHostKeyChecking=no",
+            "-o","UserKnownHostsFile=/dev/null",
+            "-o","ConnectTimeout=15",
+            "-l",access_profile.user]
         if access_profile.port and access_profile.port!=22:
             cmd_args+=["-p",str(access_profile.port)]
         cmd_args+=[access_profile.address]
         CLI.__init__(self,profile,access_profile)
         PTYSocket.__init__(self,factory,cmd_args)
         ScriptSocket.__init__(self)
+    
+    def is_stale(self):
+        self.async_check_fsm()
+        return PTYSocket.is_stale(self)
 ##
 ##
 ##
